@@ -22,6 +22,7 @@
 #include <config.h>
 #include <gtk/gtk.h>
 #include <gthumb.h>
+#include "dlg-bookmarks.h"
 
 
 typedef struct {
@@ -340,7 +341,7 @@ dlg_bookmarks (GthBrowser *browser)
 
 	data = g_new0 (DialogData, 1);
 	data->browser = browser;
-	data->builder = _gtk_builder_new_from_file ("bookmarks.ui", "bookmarks");
+	data->builder = gtk_builder_new_from_resource ("/org/gnome/gThumb/bookmarks/data/ui/bookmarks.ui");
 	data->last_selected_uri = NULL;
 	data->entry_changed = FALSE;
 
@@ -358,6 +359,7 @@ dlg_bookmarks (GthBrowser *browser)
 
 	data->uri_list = gth_uri_list_new ();
 	gtk_widget_show (data->uri_list);
+	gtk_widget_set_vexpand (data->uri_list, TRUE);
 	gtk_container_add (GTK_CONTAINER (bm_list_container), data->uri_list);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (bm_bookmarks_label), data->uri_list);
 
