@@ -25,6 +25,9 @@
 #include <stdlib.h>
 #include <setjmp.h>
 #include <jpeglib.h>
+#if HAVE_LCMS2
+#include <lcms2.h>
+#endif
 #include <gthumb.h>
 #include <extensions/jpeg_utils/jmemorysrc.h>
 #include <extensions/jpeg_utils/jpeg-info.h>
@@ -155,6 +158,7 @@ _cairo_image_surface_create_from_jpeg (GInputStream  *istream,
 				       GError       **error)
 {
 	GthImage                      *image;
+	JpegInfoFlags		       info_flags;
 	gboolean                       load_scaled;
 	GthTransform                   orientation;
 	int                            destination_width;
@@ -164,10 +168,12 @@ _cairo_image_surface_create_from_jpeg (GInputStream  *istream,
 	int                            pixel_step;
 	void                          *in_buffer;
 	gsize                          in_buffer_size;
+	JpegInfoData                   jpeg_info;
 	struct error_handler_data      jsrcerr;
 	struct jpeg_decompress_struct  srcinfo;
 	cairo_surface_t               *surface;
 	cairo_surface_metadata_t      *metadata;
+	unsigned char                 *surface_data;
 	unsigned char                 *surface_row;
 	JSAMPARRAY                     buffer;
 	int                            buffer_stride;
@@ -191,6 +197,22 @@ _cairo_image_surface_create_from_jpeg (GInputStream  *istream,
 		return image;
 	}
 
+	_jpeg_info_data_init (&jpeg_info);
+	info_flags = _JPEG_INFO_EXIF_ORIENTATION;
+#if HAVE_LCMS2
+	info_flags |= _JPEG_INFO_ICC_PROFILE;
+#endif
+	_jpeg_info_get_from_buffer (in_buffer, in_buffer_size, info_flags, &jpeg_info);
+	if (jpeg_info.valid & _JPEG_INFO_EXIF_ORIENTATION)
+		orientation = jpeg_info.orientation;
+	else
+		orientation = GTH_TRANSFORM_NONE;
+#if HAVE_LCMS2
+	if (jpeg_info.valid & _JPEG_INFO_ICC_PROFILE)
+		gth_image_set_icc_profile (image, cmsOpenProfileFromMem (jpeg_info.icc_data, jpeg_info.icc_data_size));
+#endif
+	_jpeg_info_data_dispose (&jpeg_info);
+
 	srcinfo.err = jpeg_std_error (&(jsrcerr.pub));
 	jsrcerr.pub.error_exit = fatal_error_handler;
 	jsrcerr.pub.output_message = output_message_handler;
@@ -233,7 +255,6 @@ _cairo_image_surface_create_from_jpeg (GInputStream  *istream,
 
 	jpeg_start_decompress (&srcinfo);
 
-	orientation = _jpeg_exif_orientation (in_buffer, in_buffer_size);
 	_cairo_image_surface_transform_get_steps (CAIRO_FORMAT_ARGB32,
 						  MIN (srcinfo.output_width, CAIRO_MAX_IMAGE_SIZE),
 						  MIN (srcinfo.output_height, CAIRO_MAX_IMAGE_SIZE),
@@ -262,8 +283,9 @@ _cairo_image_surface_create_from_jpeg (GInputStream  *istream,
 	}
 
 	metadata = _cairo_image_surface_get_metadata (surface);
-	metadata->has_alpha = FALSE;
-	surface_row = _cairo_image_surface_flush_and_get_data (surface) + line_start;
+	_cairo_metadata_set_has_alpha (metadata, FALSE);
+	surface_data = _cairo_image_surface_flush_and_get_data (surface);
+	surface_row = surface_data + line_start;
 
 	switch (srcinfo.out_color_space) {
 	case JCS_CMYK:
@@ -532,8 +554,7 @@ _cairo_image_surface_create_from_jpeg (GInputStream  *istream,
 			original_height = srcinfo.image_height;
 		}
 
-		metadata->original_width = original_width;
-		metadata->original_height = original_height;
+		_cairo_metadata_set_original_size (metadata, original_width, original_height);
 
 		if (original_width_p != NULL)
 			*original_width_p = original_width;
