@@ -161,7 +161,7 @@ screenshot_ready_cb (GdkPixbuf *pixbuf,
 			  "completed",
 			  G_CALLBACK (save_screenshot_task_completed_cb),
 			  save_data);
-	gth_browser_exec_task (GTH_BROWSER (save_data->browser), task, FALSE);
+	gth_browser_exec_task (GTH_BROWSER (save_data->browser), task, GTH_TASK_FLAGS_IGNORE_ERROR);
 
 }
 
