@@ -30,10 +30,10 @@
 
 #define UPDATE_QUALITY_DELAY 200
 #define UPDATE_VISIBILITY_DELAY 100
-#define N_HEADER_BAR_BUTTONS 4
+#define N_HEADER_BAR_BUTTONS 5
 #define HIDE_OVERVIEW_TIMEOUT 2 /* in seconds */
 #define OVERLAY_MARGIN 10
-#define APPLY_ICC_PROFILE_BUTTON 3
+#define APPLY_ICC_PROFILE_BUTTON 4
 #undef ALWAYS_LOAD_ORIGINAL_SIZE
 
 
@@ -53,6 +53,7 @@ static const GActionEntry actions[] = {
 	{ "image-zoom-100", gth_browser_activate_image_zoom_100 },
 	{ "image-zoom-fit", gth_browser_activate_image_zoom_fit },
 	{ "image-zoom-fit-width", gth_browser_activate_image_zoom_fit_width },
+	{ "image-zoom-fit-height", gth_browser_activate_image_zoom_fit_height },
 	{ "image-undo", gth_browser_activate_image_undo },
 	{ "image-redo", gth_browser_activate_image_redo },
 	{ "copy-image", gth_browser_activate_copy_image },
@@ -150,12 +151,7 @@ _gth_image_preloader_get_requested_size_for_next_images (GthImageViewerPage *sel
 	case GTH_ZOOM_CHANGE_ACTUAL_SIZE:
 		requested_size = -1;
 		break;
-
-	case GTH_ZOOM_CHANGE_KEEP_PREV:
-	case GTH_ZOOM_CHANGE_FIT_SIZE:
-	case GTH_ZOOM_CHANGE_FIT_SIZE_IF_LARGER:
-	case GTH_ZOOM_CHANGE_FIT_WIDTH:
-	case GTH_ZOOM_CHANGE_FIT_WIDTH_IF_LARGER:
+	default:
 		requested_size = get_viewer_size (self);
 		break;
 	}
@@ -186,10 +182,7 @@ _gth_image_preloader_get_requested_size_for_current_image (GthImageViewerPage *s
 			requested_size = -1;
 		break;
 
-	case GTH_FIT_SIZE:
-	case GTH_FIT_SIZE_IF_LARGER:
-	case GTH_FIT_WIDTH:
-	case GTH_FIT_WIDTH_IF_LARGER:
+	default:
 		requested_size = get_viewer_size (self);
 		break;
 	}
@@ -369,10 +362,8 @@ hide_overview_after_timeout (gpointer data)
 		g_source_remove (self->priv->hide_overview_id);
 	self->priv->hide_overview_id = 0;
 
-	if (! self->priv->pointer_on_overview) {
-		if (gtk_revealer_get_child_revealed (GTK_REVEALER (self->priv->overview_revealer)))
-			gtk_revealer_set_reveal_child (GTK_REVEALER (self->priv->overview_revealer), FALSE);
-	}
+	if (! self->priv->pointer_on_overview)
+		gtk_revealer_set_reveal_child (GTK_REVEALER (self->priv->overview_revealer), FALSE);
 
 	return FALSE;
 }
@@ -386,14 +377,15 @@ update_overview_visibility_now (gpointer user_data)
 	gboolean            revealed;
 
 	self = GTH_IMAGE_VIEWER_PAGE (user_data);
-	if (! self->priv->active)
-		return FALSE;
 
 	if (self->priv->update_visibility_id != 0) {
 		g_source_remove (self->priv->update_visibility_id);
 		self->priv->update_visibility_id = 0;
 	}
 
+	if (! self->priv->active)
+		return FALSE;
+
 	overview_visible = self->priv->pointer_on_overview || (self->priv->pointer_on_viewer && gth_image_viewer_has_scrollbars (GTH_IMAGE_VIEWER (self->priv->viewer)));
 	gtk_revealer_set_reveal_child (GTK_REVEALER (self->priv->overview_revealer), overview_visible);
 
@@ -410,17 +402,14 @@ update_overview_visibility_now (gpointer user_data)
 static void
 update_overview_visibility (GthImageViewerPage *self)
 {
-	if (gtk_revealer_get_child_revealed (GTK_REVEALER (self->priv->overview_revealer))) {
-		if (self->priv->update_visibility_id != 0) {
-			g_source_remove (self->priv->update_visibility_id);
-			self->priv->update_visibility_id = 0;
-		}
+	if (self->priv->update_visibility_id != 0) {
+		g_source_remove (self->priv->update_visibility_id);
+		self->priv->update_visibility_id = 0;
 	}
 
-	if (self->priv->update_visibility_id == 0)
-		self->priv->update_visibility_id = g_timeout_add (UPDATE_VISIBILITY_DELAY,
-								  update_overview_visibility_now,
-								  self);
+	self->priv->update_visibility_id = g_timeout_add (UPDATE_VISIBILITY_DELAY,
+							  update_overview_visibility_now,
+							  self);
 }
 
 
@@ -859,6 +848,13 @@ gth_image_viewer_page_real_activate (GthViewerPage *base,
 							   _("Fit to width"),
 							   "win.image-zoom-fit-width",
 							   NULL);
+	self->priv->buttons[3] =
+			gth_browser_add_header_bar_button (browser,
+							   GTH_BROWSER_HEADER_SECTION_VIEWER_VIEW,
+							   "view-zoom-fit-height-symbolic",
+							   _("Fit to height"),
+							   "win.image-zoom-fit-height",
+							   NULL);
 	self->priv->buttons[APPLY_ICC_PROFILE_BUTTON] =
 			gth_browser_add_header_bar_toggle_button (browser,
 							   	  GTH_BROWSER_HEADER_SECTION_VIEWER_TOOLS,
@@ -1253,6 +1249,7 @@ gth_image_viewer_page_real_update_sensitivity (GthViewerPage *base)
 	fit_mode = gth_image_viewer_get_fit_mode (GTH_IMAGE_VIEWER (self->priv->viewer));
 	gth_window_enable_action (GTH_WINDOW (self->priv->browser), "image-zoom-fit", zoom_enabled && (fit_mode != GTH_FIT_SIZE));
 	gth_window_enable_action (GTH_WINDOW (self->priv->browser), "image-zoom-fit-width", zoom_enabled && (fit_mode != GTH_FIT_WIDTH));
+	gth_window_enable_action (GTH_WINDOW (self->priv->browser), "image-zoom-fit-height", zoom_enabled && (fit_mode != GTH_FIT_HEIGHT));
 
 	image = gth_image_viewer_get_image (GTH_IMAGE_VIEWER (self->priv->viewer));
 	gtk_widget_set_visible (self->priv->buttons[APPLY_ICC_PROFILE_BUTTON], (image != NULL) && (gth_image_get_icc_profile (image) != NULL));
@@ -1377,7 +1374,7 @@ _gth_image_viewer_page_real_save (GthViewerPage *base,
 			  "completed",
 			  G_CALLBACK (save_image_task_completed_cb),
 			  data);
-	gth_browser_exec_task (GTH_BROWSER (self->priv->browser), task, FALSE);
+	gth_browser_exec_task (GTH_BROWSER (self->priv->browser), task, GTH_TASK_FLAGS_DEFAULT);
 
 	_g_object_unref (task);
 }
@@ -1859,7 +1856,7 @@ gth_image_viewer_page_copy_image (GthImageViewerPage *self)
 			  "completed",
 			  G_CALLBACK (copy_image_original_image_ready_cb),
 			  self);
-	gth_browser_exec_task (self->priv->browser, task, FALSE);
+	gth_browser_exec_task (self->priv->browser, task, GTH_TASK_FLAGS_DEFAULT);
 }
 
 
