@@ -25,6 +25,19 @@
 #include "gth-script.h"
 
 
+static void gth_script_dom_domizable_interface_init (DomDomizableInterface *iface);
+static void gth_script_gth_duplicable_interface_init (GthDuplicableInterface *iface);
+
+
+G_DEFINE_TYPE_WITH_CODE (GthScript,
+			 gth_script,
+			 G_TYPE_OBJECT,
+			 G_IMPLEMENT_INTERFACE (DOM_TYPE_DOMIZABLE,
+					        gth_script_dom_domizable_interface_init)
+		         G_IMPLEMENT_INTERFACE (GTH_TYPE_DUPLICABLE,
+		        		        gth_script_gth_duplicable_interface_init))
+
+
 enum {
         PROP_0,
         PROP_ID,
@@ -50,9 +63,6 @@ struct _GthScriptPrivate {
 };
 
 
-static gpointer *parent_class = NULL;
-
-
 static DomElement*
 gth_script_real_create_element (DomDomizable *base,
 				DomDocument  *doc)
@@ -85,7 +95,7 @@ _gdk_keyval_from_name (const gchar *keyval_name)
 	if (keyval_name != NULL)
 		return gdk_keyval_from_name (keyval_name);
 	else
-		return GDK_VoidSymbol;
+		return GDK_KEY_VoidSymbol;
 }
 
 
@@ -111,7 +121,7 @@ gth_script_real_load_from_element (DomDomizable *base,
 }
 
 
-GObject *
+static GObject *
 gth_script_real_duplicate (GthDuplicable *duplicable)
 {
 	GthScript *script = GTH_SCRIPT (duplicable);
@@ -139,12 +149,11 @@ gth_script_finalize (GObject *base)
 	GthScript *self;
 
 	self = GTH_SCRIPT (base);
-
 	g_free (self->priv->id);
 	g_free (self->priv->display_name);
 	g_free (self->priv->command);
 
-	G_OBJECT_CLASS (parent_class)->finalize (base);
+	G_OBJECT_CLASS (gth_script_parent_class)->finalize (base);
 }
 
 
@@ -245,7 +254,6 @@ gth_script_class_init (GthScriptClass *klass)
 {
 	GObjectClass *object_class;
 
-	parent_class = g_type_class_peek_parent (klass);
 	g_type_class_add_private (klass, sizeof (GthScriptPrivate));
 
 	object_class = G_OBJECT_CLASS (klass);
@@ -311,13 +319,13 @@ gth_script_class_init (GthScriptClass *klass)
 							    "The keyboard shortcut to activate the script",
 							    0,
 							    G_MAXUINT,
-							    GDK_VoidSymbol,
+							    GDK_KEY_VoidSymbol,
 							    G_PARAM_READWRITE));
 }
 
 
 static void
-gth_script_dom_domizable_interface_init (DomDomizableIface *iface)
+gth_script_dom_domizable_interface_init (DomDomizableInterface *iface)
 {
 	iface->create_element = gth_script_real_create_element;
 	iface->load_from_element = gth_script_real_load_from_element;
@@ -325,7 +333,7 @@ gth_script_dom_domizable_interface_init (DomDomizableIface *iface)
 
 
 static void
-gth_script_gth_duplicable_interface_init (GthDuplicableIface *iface)
+gth_script_gth_duplicable_interface_init (GthDuplicableInterface *iface)
 {
 	iface->duplicate = gth_script_real_duplicate;
 }
@@ -341,46 +349,6 @@ gth_script_init (GthScript *self)
 }
 
 
-GType
-gth_script_get_type (void)
-{
-        static GType type = 0;
-
-        if (! type) {
-                GTypeInfo type_info = {
-			sizeof (GthScriptClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) gth_script_class_init,
-			NULL,
-			NULL,
-			sizeof (GthScript),
-			0,
-			(GInstanceInitFunc) gth_script_init
-		};
-		static const GInterfaceInfo dom_domizable_info = {
-			(GInterfaceInitFunc) gth_script_dom_domizable_interface_init,
-			(GInterfaceFinalizeFunc) NULL,
-			NULL
-		};
-		static const GInterfaceInfo gth_duplicable_info = {
-			(GInterfaceInitFunc) gth_script_gth_duplicable_interface_init,
-			(GInterfaceFinalizeFunc) NULL,
-			NULL
-		};
-
-		type = g_type_register_static (G_TYPE_OBJECT,
-					       "GthScript",
-					       &type_info,
-					       0);
-		g_type_add_interface_static (type, DOM_TYPE_DOMIZABLE, &dom_domizable_info);
-		g_type_add_interface_static (type, GTH_TYPE_DUPLICABLE, &gth_duplicable_info);
-	}
-
-        return type;
-}
-
-
 GthScript*
 gth_script_new (void)
 {
@@ -594,20 +562,25 @@ thumb_loader_ready_cb (GObject      *source_object,
 		       GAsyncResult *result,
 		       gpointer      user_data)
 {
-	GtkBuilder *builder = user_data;
-	GdkPixbuf  *pixbuf;
+	GtkBuilder      *builder = user_data;
+	cairo_surface_t *image;
 
 	if (! gth_thumb_loader_load_finish (GTH_THUMB_LOADER (source_object),
 		  	 	 	    result,
-		  	 	 	    &pixbuf,
+		  	 	 	    &image,
 		  	 	 	    NULL))
 	{
 		return;
 	}
 
-	if (pixbuf != NULL) {
+	if (image != NULL) {
+		GdkPixbuf *pixbuf;
+
+		pixbuf = _gdk_pixbuf_new_from_cairo_surface (image);
 		gtk_image_set_from_pixbuf (GTK_IMAGE (_gtk_builder_get_widget (builder, "request_image")), pixbuf);
+
 		g_object_unref (pixbuf);
+		cairo_surface_destroy (image);
 	}
 
 	g_object_unref (builder);
@@ -645,7 +618,7 @@ ask_value (ReplaceData  *replace_data,
 	else
 		default_value = "";
 
-	builder = _gtk_builder_new_from_file ("ask-value.ui", "list_tools");
+	builder = gtk_builder_new_from_resource ("/org/gnome/gThumb/list_tools/data/ui/ask-value.ui");
 	dialog = _gtk_builder_get_widget (builder, "ask_value_dialog");
 	gtk_label_set_text (GTK_LABEL (_gtk_builder_get_widget (builder, "title_label")), gth_script_get_display_name (replace_data->script));
 	gtk_label_set_text (GTK_LABEL (_gtk_builder_get_widget (builder, "filename_label")), g_file_info_get_display_name (file_data->info));
