@@ -92,10 +92,11 @@ struct _GthSlideshowPrivate {
 
 
 static void
-_gth_slideshow_close (GthSlideshow *self)
+_gth_slideshow_close_cb (gpointer user_data)
 {
-	gboolean    close_browser;
-	GthBrowser *browser;
+	GthSlideshow *self = user_data;
+	gboolean      close_browser;
+	GthBrowser   *browser;
 
 	browser = self->priv->browser;
 	close_browser = ! gtk_widget_get_visible (GTK_WIDGET (browser));
@@ -108,6 +109,13 @@ _gth_slideshow_close (GthSlideshow *self)
 }
 
 
+static void
+_gth_slideshow_close (GthSlideshow *self)
+{
+	call_when_idle (_gth_slideshow_close_cb, self);
+}
+
+
 static int
 shuffle_func (gconstpointer a,
 	      gconstpointer b)
