@@ -323,6 +323,11 @@ gboolean        _g_file_attributes_matches_any_v (const char *attributes,
 void            _g_file_info_swap_attributes     (GFileInfo  *info,
 						  const char *attr1,
 						  const char *attr2);
+void            _g_file_info_set_secondary_sort_order
+						 (GFileInfo  *info,
+						  gint32      sort_order);
+gint32          _g_file_info_get_secondary_sort_order
+						 (GFileInfo  *info);
 const char *    _g_content_type_guess_from_name  (const char *filename);
 gboolean        _g_content_type_is_a             (const char *type,
 						  const char *supertype);
