@@ -130,8 +130,8 @@ gth_empty_list_realize (GtkWidget *widget)
 	window = gdk_window_new (gtk_widget_get_parent_window (widget),
 			         &attributes,
 			         attributes_mask);
+	gtk_widget_register_window (widget, window);
 	gtk_widget_set_window (widget, window);
-	gdk_window_set_user_data (window, widget);
 
 	/* Style */
 
@@ -213,6 +213,12 @@ gth_empty_list_draw (GtkWidget *widget,
 
 	gtk_widget_get_allocation (widget, &allocation);
 
+	gtk_render_background (style_context, cr,
+			       0,
+			       0,
+			       allocation.width,
+			       allocation.height);
+
 	gtk_render_frame (style_context, cr,
 			  0,
 			  0,
@@ -220,17 +226,18 @@ gth_empty_list_draw (GtkWidget *widget,
 			  allocation.height);
 
 	if (self->priv->text != NULL) {
-		PangoRectangle bounds;
-		GdkRGBA        color;
+		PangoRectangle        bounds;
+		GdkRGBA               color;
+		PangoFontDescription *font;
 
 		pango_layout_set_width (self->priv->layout, allocation.width * PANGO_SCALE);
 		pango_layout_set_text (self->priv->layout, self->priv->text, strlen (self->priv->text));
 		pango_layout_get_pixel_extents (self->priv->layout, NULL, &bounds);
-		pango_layout_set_font_description (self->priv->layout, gtk_style_context_get_font (style_context, gtk_widget_get_state (widget)));
-		gtk_style_context_get_color (style_context, gtk_widget_get_state (widget), &color);
+		gtk_style_context_get (style_context, gtk_widget_get_state_flags (widget), "font", &font, NULL);
+		pango_layout_set_font_description (self->priv->layout, font);
+		gtk_style_context_get_color (style_context, gtk_widget_get_state_flags (widget), &color);
 		gdk_cairo_set_source_rgba (cr, &color);
-		cairo_move_to (cr, 0, (allocation.height - bounds.height) / 2);
-		pango_cairo_layout_path (cr, self->priv->layout);
+		gtk_render_layout (style_context, cr, 0, (allocation.height - bounds.height) / 2, self->priv->layout);
 		cairo_fill (cr);
 	}
 
