@@ -299,7 +299,7 @@ gth_extension_module_real_is_configurable (GthExtension *base)
 	GthExtensionModule *self;
 	char               *function_name;
 	gboolean            result = FALSE;
-	gboolean (*is_configurable_func) ();
+	gboolean (*is_configurable_func) (void);
 
 	g_return_val_if_fail (GTH_IS_EXTENSION_MODULE (base), FALSE);
 
@@ -448,6 +448,7 @@ gth_extension_description_load_from_file (GthExtensionDescription *desc,
 {
 	GKeyFile *key_file;
 	char     *file_path;
+	char     *api;
 	char     *basename;
 
 	key_file = g_key_file_new ();
@@ -458,6 +459,14 @@ gth_extension_description_load_from_file (GthExtensionDescription *desc,
 		return FALSE;
 	}
 
+	api = g_key_file_get_string (key_file, "Loader", "API", NULL);
+	if (g_strcmp0 (api, GTHUMB_API_VERSION) != 0) {
+		g_free (api);
+		g_free (file_path);
+		g_key_file_free (key_file);
+		return FALSE;
+	}
+
 	basename = g_file_get_basename (file);
 	desc->id = _g_uri_remove_extension (basename);
 	desc->name = g_key_file_get_locale_string (key_file, "Extension", "Name", NULL, NULL);
@@ -476,6 +485,7 @@ gth_extension_description_load_from_file (GthExtensionDescription *desc,
 	desc->loader_after = g_key_file_get_string_list (key_file, "Loader", "After", NULL, NULL);
 
 	g_free (basename);
+	g_free (api);
 	g_free (file_path);
 	g_key_file_free (key_file);
 
