@@ -526,7 +526,7 @@ thumbnail_job_cancel (ThumbnailJob *job)
 /* --- */
 
 
-static void
+static gboolean
 vadj_changed_cb (GtkAdjustment *adjustment,
 		 gpointer       user_data)
 {
@@ -534,6 +534,8 @@ vadj_changed_cb (GtkAdjustment *adjustment,
 
 	file_list->priv->thumbnailer_state.phase = THUMBNAILER_PHASE_INITIALIZE;
 	start_update_next_thumb (GTH_FILE_LIST (user_data));
+
+	return FALSE;
 }
 
 
@@ -636,6 +638,7 @@ gth_file_list_construct (GthFileList     *file_list,
 	/* the file view */
 
 	file_list->priv->scrolled_window = gtk_scrolled_window_new (NULL, NULL);
+	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (file_list->priv->scrolled_window), GTK_SHADOW_ETCHED_IN);
 
 	file_list->priv->vadj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (file_list->priv->scrolled_window));
 	g_signal_connect (G_OBJECT (file_list->priv->vadj),
@@ -660,30 +663,13 @@ gth_file_list_construct (GthFileList     *file_list,
 			  "rows-reordered",
 			  G_CALLBACK (file_store_rows_reordered_cb),
 			  file_list);
+	g_signal_connect (G_OBJECT (file_list->priv->view),
+			  "drag-data-get",
+			  G_CALLBACK (file_view_drag_data_get_cb),
+			  file_list);
 
-	if (enable_drag_drop) {
-		GtkTargetList  *target_list;
-		GtkTargetEntry *targets;
-		int             n_targets;
-
-		target_list = gtk_target_list_new (NULL, 0);
-		gtk_target_list_add_uri_targets (target_list, 0);
-		gtk_target_list_add_text_targets (target_list, 0);
-		targets = gtk_target_table_new_from_list (target_list, &n_targets);
-		gth_file_view_enable_drag_source (GTH_FILE_VIEW (file_list->priv->view),
-						  GDK_BUTTON1_MASK | GDK_BUTTON2_MASK,
-						  targets,
-						  n_targets,
-						  GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_ASK);
-
-		gtk_target_list_unref (target_list);
-		gtk_target_table_free (targets, n_targets);
-
-		g_signal_connect (G_OBJECT (file_list->priv->view),
-				  "drag-data-get",
-				  G_CALLBACK (file_view_drag_data_get_cb),
-				  file_list);
-	}
+	if (enable_drag_drop)
+		gth_file_list_enable_drag_source (file_list, GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_ASK);
 
 	gth_file_list_set_mode (file_list, list_type);
 
@@ -1697,6 +1683,11 @@ update_thumbs_stopped (gpointer callback_data)
 {
 	GthFileList *file_list = callback_data;
 
+	if (file_list->priv->update_event != 0) {
+		g_source_remove (file_list->priv->update_event);
+		file_list->priv->update_event = 0;
+	}
+
 	file_list->priv->loading_thumbs = FALSE;
 	_gth_file_list_exec_next_op (file_list);
 
@@ -2145,3 +2136,33 @@ gth_file_list_prev_file (GthFileList *file_list,
 
 	return (scan != NULL) ? pos : -1;
 }
+
+
+void
+gth_file_list_enable_drag_source (GthFileList     *file_list,
+				  GdkDragAction    actions)
+{
+	GtkTargetList  *target_list;
+	GtkTargetEntry *targets;
+	int             n_targets;
+
+	target_list = gtk_target_list_new (NULL, 0);
+	gtk_target_list_add_uri_targets (target_list, 0);
+	gtk_target_list_add_text_targets (target_list, 0);
+	targets = gtk_target_table_new_from_list (target_list, &n_targets);
+	gth_file_view_enable_drag_source (GTH_FILE_VIEW (file_list->priv->view),
+					  GDK_BUTTON1_MASK | GDK_BUTTON2_MASK,
+					  targets,
+					  n_targets,
+					  actions);
+
+	gtk_target_list_unref (target_list);
+	gtk_target_table_free (targets, n_targets);
+}
+
+
+void
+gth_file_list_unset_drag_source (GthFileList *file_list)
+{
+	gth_file_view_unset_drag_source (GTH_FILE_VIEW (file_list->priv->view));
+}
