@@ -21,9 +21,11 @@
 
 #include <config.h>
 #include <glib/gi18n.h>
+#include "cairo-utils.h"
 #include "glib-utils.h"
 #include "gth-file-store.h"
 #include "gth-marshal.h"
+#include "gth-string-list.h"
 
 
 #undef  DEBUG_FILE_STORE
@@ -43,11 +45,10 @@ static guint gth_file_store_signals[LAST_SIGNAL] = { 0 };
 
 
 typedef struct {
-	int          ref_count;
-
-	GthFileData *file_data;
-	GdkPixbuf   *thumbnail;
-	gboolean     is_icon : 1;
+	int              ref_count;
+	GthFileData     *file_data;
+	cairo_surface_t *thumbnail;
+	gboolean         is_icon : 1;
 
 	/*< private >*/
 
@@ -114,19 +115,19 @@ _gth_file_row_set_file (GthFileRow  *row,
 
 
 static void
-_gth_file_row_set_thumbnail (GthFileRow *row,
-			     GdkPixbuf  *thumbnail)
+_gth_file_row_set_thumbnail (GthFileRow      *row,
+			     cairo_surface_t *thumbnail)
 {
 	if (thumbnail != NULL) {
-		g_object_ref (thumbnail);
+		cairo_surface_reference (thumbnail);
 		if (row->thumbnail != NULL)
-			g_object_unref (row->thumbnail);
+			cairo_surface_destroy (row->thumbnail);
 		row->thumbnail = thumbnail;
 	}
 }
 
 
-GthFileRow *
+static GthFileRow *
 _gth_file_row_copy (GthFileRow *row)
 {
 	GthFileRow *row2;
@@ -161,7 +162,7 @@ _gth_file_row_unref (GthFileRow *row)
 	if (row->file_data != NULL)
 		g_object_unref (row->file_data);
 	if (row->thumbnail != NULL)
-		g_object_unref (row->thumbnail);
+		cairo_surface_destroy (row->thumbnail);
 	g_free (row);
 }
 
@@ -262,8 +263,9 @@ gth_file_store_init (GthFileStore *file_store)
 
 	if (column_type[0] == G_TYPE_INVALID) {
 		column_type[GTH_FILE_STORE_FILE_DATA_COLUMN] = GTH_TYPE_FILE_DATA;
-		column_type[GTH_FILE_STORE_THUMBNAIL_COLUMN] = GDK_TYPE_PIXBUF;
+		column_type[GTH_FILE_STORE_THUMBNAIL_COLUMN] = GTH_TYPE_CAIRO_SURFACE;
 		column_type[GTH_FILE_STORE_IS_ICON_COLUMN] = G_TYPE_BOOLEAN;
+		column_type[GTH_FILE_STORE_EMBLEMS_COLUMN] = GTH_TYPE_STRING_LIST;
 	}
 }
 
@@ -368,13 +370,17 @@ gth_file_store_get_value (GtkTreeModel *tree_model,
 		g_value_set_object (value, row->file_data);
 		break;
 	case GTH_FILE_STORE_THUMBNAIL_COLUMN:
-		g_value_init (value, GDK_TYPE_PIXBUF);
-		g_value_set_object (value, row->thumbnail);
+		g_value_init (value, GTH_TYPE_CAIRO_SURFACE);
+		g_value_set_boxed (value, row->thumbnail);
 		break;
 	case GTH_FILE_STORE_IS_ICON_COLUMN:
 		g_value_init (value, G_TYPE_BOOLEAN);
 		g_value_set_boolean (value, row->is_icon);
 		break;
+	case GTH_FILE_STORE_EMBLEMS_COLUMN:
+		g_value_init (value, G_TYPE_STRING);
+		g_value_set_object (value, g_file_info_get_attribute_object (row->file_data->info, GTH_FILE_ATTRIBUTE_EMBLEMS));
+		break;
 	}
 }
 
@@ -628,16 +634,6 @@ gth_file_store_new (void)
 }
 
 
-int
-gth_file_store_get_abs_pos (GthFileStore *file_store,
-			    int           pos)
-{
-	g_return_val_if_fail ((pos >= 0) && (pos < file_store->priv->num_rows), -1);
-
-	return file_store->priv->rows[pos]->abs_pos;
-}
-
-
 static void
 _gth_file_store_increment_stamp (GthFileStore *file_store)
 {
@@ -1344,11 +1340,11 @@ gth_file_store_get_prev_visible (GthFileStore *file_store,
 
 
 void
-gth_file_store_add (GthFileStore *file_store,
-		    GthFileData  *file,
-		    GdkPixbuf    *thumbnail,
-		    gboolean      is_icon,
-		    int           position)
+gth_file_store_add (GthFileStore    *file_store,
+		    GthFileData     *file,
+		    cairo_surface_t *thumbnail,
+		    gboolean         is_icon,
+		    int              position)
 {
 	gth_file_store_queue_add (file_store, file, thumbnail, is_icon);
 	gth_file_store_exec_add (file_store, position);
@@ -1356,10 +1352,10 @@ gth_file_store_add (GthFileStore *file_store,
 
 
 void
-gth_file_store_queue_add (GthFileStore *file_store,
-			  GthFileData  *file,
-			  GdkPixbuf    *thumbnail,
-			  gboolean      is_icon)
+gth_file_store_queue_add (GthFileStore    *file_store,
+			  GthFileData     *file,
+			  cairo_surface_t *thumbnail,
+			  gboolean         is_icon)
 {
 	GthFileRow *row;
 
@@ -1397,8 +1393,9 @@ gth_file_store_queue_set_valist (GthFileStore *file_store,
 
  	column = va_arg (var_args, int);
   	while (column != -1) {
-  		GthFileData *file_data;
-  		GdkPixbuf   *thumbnail;
+  		GthFileData     *file_data;
+  		cairo_surface_t *thumbnail;
+  		GthStringList   *string_list;
 
   		switch (column) {
   		case GTH_FILE_STORE_FILE_DATA_COLUMN:
@@ -1409,8 +1406,7 @@ gth_file_store_queue_set_valist (GthFileStore *file_store,
   			file_store->priv->update_filter = TRUE;
   			break;
   		case GTH_FILE_STORE_THUMBNAIL_COLUMN:
-  			thumbnail = va_arg (var_args, GdkPixbuf *);
-  			g_return_if_fail (GDK_IS_PIXBUF (thumbnail));
+  			thumbnail = va_arg (var_args, cairo_surface_t *);
   			_gth_file_row_set_thumbnail (row, thumbnail);
   			row->changed = TRUE;
   			break;
@@ -1418,6 +1414,11 @@ gth_file_store_queue_set_valist (GthFileStore *file_store,
   			row->is_icon = va_arg (var_args, gboolean);
   			row->changed = TRUE;
   			break;
+  		case GTH_FILE_STORE_EMBLEMS_COLUMN:
+  			string_list = va_arg (var_args, GthStringList *);
+  			g_file_info_set_attribute_object (row->file_data->info, GTH_FILE_ATTRIBUTE_EMBLEMS, G_OBJECT (string_list));
+  			row->changed = TRUE;
+  			break;
   		default:
   			g_warning ("%s: Invalid column number %d added to iter (remember to end your list of columns with a -1)", G_STRLOC, column);
   			break;
