@@ -24,6 +24,7 @@
 
 #include <gtk/gtk.h>
 #include "gth-toolbox.h"
+#include "gth-browser.h"
 
 G_BEGIN_DECLS
 
@@ -53,6 +54,9 @@ struct _GthFileToolClass {
 	void         (*cancel)             (GthFileTool *self);
 	GtkWidget *  (*get_options)        (GthFileTool *self);
 	void         (*destroy_options)    (GthFileTool *self);
+	void         (*apply_options)      (GthFileTool *self);
+	void         (*populate_headerbar) (GthFileTool *self,
+					    GthBrowser  *browser);
 
 	/*< signals >*/
 
@@ -77,6 +81,9 @@ GthToolboxSection	gth_file_tool_get_section		(GthFileTool		*self);
 void			gth_file_tool_destroy_options		(GthFileTool		*self);
 void			gth_file_tool_show_options		(GthFileTool		*self);
 void			gth_file_tool_hide_options		(GthFileTool		*self);
+void			gth_file_tool_apply_options		(GthFileTool		*self);
+void			gth_file_tool_populate_headerbar	(GthFileTool		*self,
+								 GthBrowser		*browser);
 
 G_END_DECLS
 
