@@ -26,6 +26,7 @@
 #include "cairo-scale.h"
 #include "cairo-utils.h"
 #include "glib-utils.h"
+#include "gth-image.h"
 #include "gth-image-preloader.h"
 #include "gth-image-utils.h"
 #include "gth-marshal.h"
@@ -77,6 +78,7 @@ struct _GthImagePreloaderPrivate {
 	GthImageLoader		*loader;
 	GQueue			*cache;
 	guint                    load_next_id;
+	GthICCProfile            out_profile;
 };
 
 
@@ -277,6 +279,7 @@ gth_image_preloader_init (GthImagePreloader *self)
 	self->priv->loader = gth_image_loader_new (NULL, NULL);
 	self->priv->cache = g_queue_new ();
 	self->priv->load_next_id = 0;
+	self->priv->out_profile = NULL;
 }
 
 
@@ -287,6 +290,15 @@ gth_image_preloader_new (void)
 }
 
 
+void
+gth_image_preloader_set_out_profile (GthImagePreloader *self,
+				     GthICCProfile      out_profile)
+{
+	g_return_if_fail (self != NULL);
+	self->priv->out_profile = out_profile;
+}
+
+
 /* -- gth_image_preloader_load -- */
 
 
@@ -707,6 +719,7 @@ _gth_image_preloader_load_current_file (GthImagePreloader *self,
 	g_print ("load %s @%d\n", g_file_get_uri (requested_file->file), ignore_requested_size ? -1 : request->requested_size);
 #endif
 
+	gth_image_loader_set_out_profile (self->priv->loader, self->priv->out_profile);
 	gth_image_loader_load (self->priv->loader,
 			       requested_file,
 			       ignore_requested_size ? -1 : request->requested_size,
@@ -777,7 +790,10 @@ gth_image_preloader_load (GthImagePreloader	 *self,
 	va_list      args;
 
 #ifdef DEBUG_PRELOADER
-	g_print ("request %s @%d\n", g_file_get_uri (requested->file), requested_size);
+	if (requested != NULL)
+		g_print ("request %s @%d\n", g_file_get_uri (requested->file), requested_size);
+	else
+		g_print ("request modified image @%d\n", requested_size);
 #endif
 
 	request = load_request_new (self);
@@ -850,7 +866,7 @@ gth_image_preloader_load_finish (GthImagePreloader	 *self,
 
 void
 gth_image_preloader_set_modified_image (GthImagePreloader *self,
-					cairo_surface_t	  *image)
+					GthImage	  *image)
 {
 	GList     *scan;
 	CacheData *cache_data;
@@ -873,7 +889,7 @@ gth_image_preloader_set_modified_image (GthImagePreloader *self,
 
 	cache_data = cache_data_new ();
 	cache_data->file_data = GTH_MODIFIED_IMAGE;
-	cache_data->image = gth_image_new_for_surface (image);
+	cache_data->image = g_object_ref (image);
 	cache_data->original_width = -1;
 	cache_data->original_height = -1;
 	cache_data->requested_size = -1;
@@ -896,3 +912,11 @@ gth_image_preloader_get_modified_image (GthImagePreloader *self)
 
 	return NULL;
 }
+
+
+void
+gth_image_preloader_clear_cache (GthImagePreloader *self)
+{
+	g_queue_free_full (self->priv->cache, (GDestroyNotify) cache_data_unref);
+	self->priv->cache = g_queue_new ();
+}
