@@ -38,6 +38,7 @@ enum {
 	FOLDER_CONTENT_CHANGED,
 	FILE_RENAMED,
 	METADATA_CHANGED,
+	EMBLEMS_CHANGED,
 	ENTRY_POINTS_CHANGED,
 	ORDER_CHANGED,
 	LAST_SIGNAL
@@ -156,6 +157,16 @@ gth_monitor_class_init (GthMonitorClass *class)
 			      G_TYPE_NONE,
 			      1,
 			      G_TYPE_OBJECT);
+	monitor_signals[EMBLEMS_CHANGED] =
+		g_signal_new ("emblems-changed",
+			      G_TYPE_FROM_CLASS (class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GthMonitorClass, emblems_changed),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__BOXED,
+			      G_TYPE_NONE,
+			      1,
+			      G_TYPE_OBJECT_LIST);
 	monitor_signals[ENTRY_POINTS_CHANGED] =
 		g_signal_new ("entry-points-changed",
 			      G_TYPE_FROM_CLASS (class),
@@ -323,6 +334,62 @@ gth_monitor_file_renamed (GthMonitor *self,
 }
 
 
+/* -- gth_monitor_files_deleted -- */
+
+
+static void
+parent_table_value_free (gpointer key,
+			 gpointer value,
+			 gpointer user_data)
+{
+	_g_object_list_unref (value);
+}
+
+
+void
+gth_monitor_files_deleted (GthMonitor *monitor,
+			   GList      *list /* GFile list */)
+{
+	GHashTable *parent_table;
+	GList      *scan;
+	GList      *parent_list;
+
+	parent_table = g_hash_table_new_full (g_file_hash, (GEqualFunc) g_file_equal, g_object_unref, NULL);
+	for (scan = list; scan; scan = scan->next) {
+		GFile *file = scan->data;
+		GFile *parent;
+		GList *children;
+
+		parent = g_file_get_parent (file);
+		if (parent == NULL)
+			return;
+
+		children = g_hash_table_lookup (parent_table, parent);
+		children = g_list_prepend (children, g_object_ref (file));
+		g_hash_table_replace (parent_table, g_object_ref (parent), children);
+
+		g_object_unref (parent);
+	}
+
+	parent_list = g_hash_table_get_keys (parent_table);
+	for (scan = parent_list; scan; scan = scan->next) {
+		GFile *parent = scan->data;
+		GList *children;
+
+		children = g_hash_table_lookup (parent_table, parent);
+		if (children != NULL)
+			gth_monitor_folder_changed (monitor,
+						    parent,
+						    children,
+						    GTH_MONITOR_EVENT_DELETED);
+	}
+
+	g_list_free (parent_list);
+	g_hash_table_foreach (parent_table, parent_table_value_free, NULL);
+	g_hash_table_unref (parent_table);
+}
+
+
 void
 gth_monitor_metadata_changed (GthMonitor  *self,
 			      GthFileData *file_data)
@@ -340,7 +407,37 @@ gth_monitor_metadata_changed (GthMonitor  *self,
 
 
 void
-gth_monitor_file_entry_points_changed (GthMonitor *self)
+gth_monitor_emblems_changed (GthMonitor *self,
+			     GList      *list /* GFile list */)
+{
+	GList *changed_list = NULL;
+	GList *scan;
+
+	/* ignore paused files */
+	for (scan = list; scan; scan = scan->next) {
+		GFile *file = scan->data;
+
+		if (g_hash_table_lookup (self->priv->paused_files, file) != NULL)
+			continue;
+
+		changed_list = g_list_prepend (changed_list, g_object_ref (file));
+	}
+	changed_list = g_list_reverse (changed_list);
+
+	if (changed_list == NULL)
+		return;
+
+	g_signal_emit (G_OBJECT (self),
+		       monitor_signals[EMBLEMS_CHANGED],
+		       0,
+		       changed_list);
+
+	_g_object_list_unref (changed_list);
+}
+
+
+void
+gth_monitor_entry_points_changed (GthMonitor *self)
 {
 	g_return_if_fail (GTH_IS_MONITOR (self));
 
