@@ -55,7 +55,9 @@ GthOpData text_op_data[] = {
 GthOpData int_op_data[] = {
 	{ N_("is lower than"), GTH_TEST_OP_LOWER, FALSE },
 	{ N_("is greater than"), GTH_TEST_OP_GREATER, FALSE },
-	{ N_("is equal to"), GTH_TEST_OP_EQUAL, FALSE }
+	{ N_("is equal to"), GTH_TEST_OP_EQUAL, FALSE },
+	{ N_("is greater than or equal to"), GTH_TEST_OP_LOWER, TRUE },
+	{ N_("is lower than or equal to"), GTH_TEST_OP_GREATER, TRUE }
 };
 
 GthOpData date_op_data[] = {
@@ -86,7 +88,8 @@ enum {
         PROP_DATA_AS_DATE,
         PROP_GET_DATA,
         PROP_OP,
-        PROP_NEGATIVE
+        PROP_NEGATIVE,
+        PROP_MAX_INT
 };
 
 
@@ -101,6 +104,7 @@ struct _GthTestSimplePrivate
 	GthTestGetData   get_data;
 	GthTestOp        op;
 	gboolean         negative;
+	gint64           max_int;
 	GPatternSpec    *pattern;
 	gboolean         has_focus;
 	GtkWidget       *text_entry;
@@ -109,12 +113,25 @@ struct _GthTestSimplePrivate
 	GtkWidget       *date_op_combo_box;
 	GtkWidget       *size_combo_box;
 	GtkWidget       *time_selector;
+	GtkWidget       *spinbutton;
 };
 
 
-static GthTestClass *parent_class = NULL;
-static DomDomizableIface* dom_domizable_parent_iface = NULL;
-static GthDuplicableIface *gth_duplicable_parent_iface = NULL;
+static DomDomizableInterface* dom_domizable_parent_iface = NULL;
+static GthDuplicableInterface *gth_duplicable_parent_iface = NULL;
+
+
+static void gth_test_simple_dom_domizable_interface_init (DomDomizableInterface * iface);
+static void gth_test_simple_gth_duplicable_interface_init (GthDuplicableInterface *iface);
+
+
+G_DEFINE_TYPE_WITH_CODE (GthTestSimple,
+			 gth_test_simple,
+			 GTH_TYPE_TEST,
+			 G_IMPLEMENT_INTERFACE (DOM_TYPE_DOMIZABLE,
+					 	gth_test_simple_dom_domizable_interface_init)
+			 G_IMPLEMENT_INTERFACE (GTH_TYPE_DUPLICABLE,
+					 	gth_test_simple_gth_duplicable_interface_init))
 
 
 static void
@@ -158,14 +175,7 @@ gth_test_simple_finalize (GObject *object)
 		test->priv = NULL;
 	}
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
-}
-
-
-static GtkWidget *
-create_control_for_integer (GthTestSimple *test)
-{
-	return NULL;
+	G_OBJECT_CLASS (gth_test_simple_parent_class)->finalize (object);
 }
 
 
@@ -216,6 +226,84 @@ size_combo_box_changed_cb (GtkComboBox   *combo_box,
 }
 
 
+static void
+spinbutton_changed_cb (GtkSpinButton *spinbutton,
+                       GthTestSimple *test)
+{
+	gth_test_update_from_control (GTH_TEST (test), NULL);
+	gth_test_changed (GTH_TEST (test));
+}
+
+
+static void
+focus_event_cb (GtkWidget     *widget,
+                GdkEvent      *event,
+                GthTestSimple *test)
+{
+	gth_test_update_from_control (GTH_TEST (test), NULL);
+	gth_test_changed (GTH_TEST (test));
+}
+
+
+static GtkWidget *
+create_control_for_integer (GthTestSimple *test)
+{
+	GtkWidget *control;
+	int        i, op_idx;
+
+	control = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+
+	/* text operation combo box */
+
+	test->priv->text_op_combo_box = gtk_combo_box_text_new ();
+	gtk_widget_show (test->priv->text_op_combo_box);
+
+	op_idx = 0;
+	for (i = 0; i < G_N_ELEMENTS (int_op_data); i++) {
+		gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (test->priv->text_op_combo_box),
+						_(int_op_data[i].name));
+		if ((int_op_data[i].op == test->priv->op) && (int_op_data[i].negative == test->priv->negative))
+			op_idx = i;
+	}
+	gtk_combo_box_set_active (GTK_COMBO_BOX (test->priv->text_op_combo_box), op_idx);
+
+	g_signal_connect (G_OBJECT (test->priv->text_op_combo_box),
+			  "changed",
+			  G_CALLBACK (size_op_combo_box_changed_cb),
+			  test);
+
+	/* spin button */
+
+	test->priv->spinbutton = gtk_spin_button_new_with_range (0, test->priv->max_int, 1);
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (test->priv->spinbutton), 0);
+	gtk_widget_show (test->priv->spinbutton);
+
+	g_signal_connect (G_OBJECT (test->priv->spinbutton),
+			  "value-changed",
+			  G_CALLBACK (spinbutton_changed_cb),
+			  test);
+	g_signal_connect (G_OBJECT (test->priv->spinbutton),
+			  "activate",
+			  G_CALLBACK (size_text_entry_activate_cb),
+			  test);
+	g_signal_connect (G_OBJECT (test->priv->spinbutton),
+			  "focus-in-event",
+			  G_CALLBACK (focus_event_cb),
+			  test);
+	g_signal_connect (G_OBJECT (test->priv->spinbutton),
+			  "focus-out-event",
+			  G_CALLBACK (focus_event_cb),
+			  test);
+
+	/**/
+
+	gtk_box_pack_start (GTK_BOX (control), test->priv->text_op_combo_box, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (control), test->priv->spinbutton, FALSE, FALSE, 0);
+
+	return control;
+}
+
+
 static GtkWidget *
 create_control_for_size (GthTestSimple *test)
 {
@@ -223,7 +311,7 @@ create_control_for_size (GthTestSimple *test)
 	int        i, op_idx, size_idx;
 	gboolean   size_set = FALSE;
 
-	control = gtk_hbox_new (FALSE, 6);
+	control = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
 
 	/* text operation combo box */
 
@@ -323,7 +411,7 @@ create_control_for_string (GthTestSimple *test)
 	GtkWidget *control;
 	int        i, op_idx;
 
-	control = gtk_hbox_new (FALSE, 6);
+	control = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
 
 	/* text operation combo box */
 
@@ -398,7 +486,7 @@ create_control_for_date (GthTestSimple *test)
 	GtkWidget *control;
 	int        i, op_idx;
 
-	control = gtk_hbox_new (FALSE, 6);
+	control = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
 
 	/* date operation combo box */
 
@@ -793,13 +881,7 @@ update_from_control_for_integer (GthTestSimple  *self,
 	op_data = int_op_data[gtk_combo_box_get_active (GTK_COMBO_BOX (self->priv->text_op_combo_box))];
 	self->priv->op = op_data.op;
 	self->priv->negative = op_data.negative;
-	gth_test_simple_set_data_as_int (self, atol (gtk_entry_get_text (GTK_ENTRY (self->priv->text_entry))));
-
-	if (self->priv->data.i == 0) {
-		if (error != NULL)
-			*error = g_error_new (GTH_TEST_ERROR, 0, _("The test definition is incomplete"));
-		return FALSE;
-	}
+	gth_test_simple_set_data_as_int (self, gtk_spin_button_get_value_as_int  (GTK_SPIN_BUTTON (self->priv->spinbutton)));
 
 	return TRUE;
 }
@@ -951,6 +1033,7 @@ gth_test_simple_real_duplicate (GthDuplicable *duplicable)
 	new_test->priv->get_data = test->priv->get_data;
 	new_test->priv->op = test->priv->op;
 	new_test->priv->negative = test->priv->negative;
+	new_test->priv->max_int = test->priv->max_int;
 
 	return (GObject *) new_test;
 }
@@ -998,6 +1081,10 @@ gth_test_simple_set_property (GObject      *object,
 		test->priv->negative = g_value_get_boolean (value);
 		break;
 
+	case PROP_MAX_INT:
+		test->priv->max_int = g_value_get_int (value);
+		break;
+
 	default:
 		break;
 	}
@@ -1043,6 +1130,10 @@ gth_test_simple_get_property (GObject    *object,
 		g_value_set_boolean (value, test->priv->negative);
 		break;
 
+	case PROP_MAX_INT:
+		g_value_set_int (value, test->priv->max_int);
+		break;
+
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 		break;
@@ -1056,13 +1147,12 @@ gth_test_simple_class_init (GthTestSimpleClass *class)
 	GObjectClass *object_class;
 	GthTestClass *test_class;
 
-	parent_class = g_type_class_peek_parent (class);
 	object_class = (GObjectClass*) class;
-	test_class = (GthTestClass *) class;
-
 	object_class->set_property = gth_test_simple_set_property;
 	object_class->get_property = gth_test_simple_get_property;
 	object_class->finalize = gth_test_simple_finalize;
+
+	test_class = (GthTestClass *) class;
 	test_class->create_control = gth_test_simple_real_create_control;
 	test_class->update_from_control = gth_test_simple_real_update_from_control;
 	test_class->match = gth_test_simple_real_match;
@@ -1121,11 +1211,21 @@ gth_test_simple_class_init (GthTestSimpleClass *class)
                                                                "Whether to negate the test result",
                                                                FALSE,
                                                                G_PARAM_READWRITE));
+
+	g_object_class_install_property (object_class,
+					 PROP_MAX_INT,
+					 g_param_spec_int ("max-int",
+                                                           "Max integer",
+                                                           "Max value for integers",
+                                                           G_MININT,
+                                                           G_MAXINT,
+                                                           0,
+                                                           G_PARAM_READWRITE));
 }
 
 
 static void
-gth_test_simple_dom_domizable_interface_init (DomDomizableIface * iface)
+gth_test_simple_dom_domizable_interface_init (DomDomizableInterface * iface)
 {
 	dom_domizable_parent_iface = g_type_interface_peek_parent (iface);
 	iface->create_element = gth_test_simple_real_create_element;
@@ -1134,7 +1234,7 @@ gth_test_simple_dom_domizable_interface_init (DomDomizableIface * iface)
 
 
 static void
-gth_test_simple_gth_duplicable_interface_init (GthDuplicableIface *iface)
+gth_test_simple_gth_duplicable_interface_init (GthDuplicableInterface *iface)
 {
 	gth_duplicable_parent_iface = g_type_interface_peek_parent (iface);
 	iface->duplicate = gth_test_simple_real_duplicate;
@@ -1148,46 +1248,6 @@ gth_test_simple_init (GthTestSimple *test)
 }
 
 
-GType
-gth_test_simple_get_type (void)
-{
-        static GType type = 0;
-
-        if (! type) {
-                GTypeInfo type_info = {
-			sizeof (GthTestSimpleClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) gth_test_simple_class_init,
-			NULL,
-			NULL,
-			sizeof (GthTestSimple),
-			0,
-			(GInstanceInitFunc) gth_test_simple_init
-		};
-		static const GInterfaceInfo dom_domizable_info = {
-			(GInterfaceInitFunc) gth_test_simple_dom_domizable_interface_init,
-			(GInterfaceFinalizeFunc) NULL,
-			NULL
-		};
-		static const GInterfaceInfo gth_duplicable_info = {
-			(GInterfaceInitFunc) gth_test_simple_gth_duplicable_interface_init,
-			(GInterfaceFinalizeFunc) NULL,
-			NULL
-		};
-
-		type = g_type_register_static (GTH_TYPE_TEST,
-					       "GthTestSimple",
-					       &type_info,
-					       0);
-		g_type_add_interface_static (type, DOM_TYPE_DOMIZABLE, &dom_domizable_info);
-		g_type_add_interface_static (type, GTH_TYPE_DUPLICABLE, &gth_duplicable_info);
-	}
-
-        return type;
-}
-
-
 void
 gth_test_simple_set_data_as_string (GthTestSimple *test,
 			            const char    *s)
@@ -1200,7 +1260,7 @@ gth_test_simple_set_data_as_string (GthTestSimple *test,
 
 void
 gth_test_simple_set_data_as_int (GthTestSimple *test,
-			         guint64        i)
+			         gint64         i)
 {
 	_gth_test_simple_free_data (test);
 	test->priv->data_type = GTH_TEST_DATA_TYPE_INT;
